@@ -12,15 +12,19 @@ and Wolter, K.M. (2007) [#w2007]_.
    Springer-Verlag New York, Inc
 """
 from __future__ import annotations
-from typing import Any, Optional, Union
 
 import math
+
+from typing import Any, Optional, Union
+
 import numpy as np
 import pandas as pd
-from samplics.utils.formats import fpc_as_dict, numpy_array, remove_nans, sample_size_dict
-from samplics.utils.types import Array, DictStrNum, Number, Series, StringNumber
+
 from scipy.stats import t as student
 
+from samplics.utils.formats import dict_to_dataframe, fpc_as_dict, numpy_array, remove_nans
+from samplics.utils.types import Array, Number, Series, StringNumber
+
 
 class _SurveyEstimator:
     """General approach for sample estimation of linear parameters."""
@@ -56,6 +60,7 @@ class _SurveyEstimator:
         self.number_strata: Any = None
         self.number_psus: Any = None
         self.degree_of_freedom: Any = None
+        self.as_factor: bool = False  # whether to treat outcome as factor.
 
     def __str__(self) -> Any:
         print(f"SAMPLICS - Estimation of {self.parameter.title()}\n")
@@ -71,12 +76,43 @@ class _SurveyEstimator:
             and isinstance(self.upper_ci, dict)
             and isinstance(self.coef_var, dict)
         ):
-            estimation["DOMAINS"] = self.domains
-            estimation[parameter] = self.point_est.values()
-            estimation["SE"] = self.stderror.values()
-            estimation["LCI"] = self.lower_ci.values()
-            estimation["UCI"] = self.upper_ci.values()
-            estimation["CV"] = self.coef_var.values()
+            if self.domains is not None and (self.parameter == "proportion" or self.as_factor):
+                domains = list()
+                levels = list()
+                point_est = list()
+                stderror = list()
+                lower_ci = list()
+                upper_ci = list()
+                coef_var = list()
+                for domain in self.domains:
+                    domains += np.repeat(domain, len(self.point_est[domain])).tolist()
+                    levels += list(self.point_est[domain].keys())
+                    point_est += list(self.point_est[domain].values())
+                    stderror += list(self.stderror[domain].values())
+                    lower_ci += list(self.lower_ci[domain].values())
+                    upper_ci += list(self.upper_ci[domain].values())
+                    coef_var += list(self.coef_var[domain].values())
+                estimation["DOMAIN"] = domains
+                estimation["LEVEL"] = levels
+                estimation[parameter] = point_est
+                estimation["SE"] = stderror
+                estimation["LCI"] = lower_ci
+                estimation["UCI"] = upper_ci
+                estimation["CV"] = coef_var
+            elif self.domains is not None:
+                estimation["DOMAIN"] = self.domains
+                estimation[parameter] = self.point_est.values()
+                estimation["SE"] = self.stderror.values()
+                estimation["LCI"] = self.lower_ci.values()
+                estimation["UCI"] = self.upper_ci.values()
+                estimation["CV"] = self.coef_var.values()
+            else:
+                estimation["LEVELS"] = self.point_est.keys()
+                estimation[parameter] = self.point_est.values()
+                estimation["SE"] = self.stderror.values()
+                estimation["LCI"] = self.lower_ci.values()
+                estimation["UCI"] = self.upper_ci.values()
+                estimation["CV"] = self.coef_var.values()
         else:
             estimation[parameter] = [self.point_est]
             estimation["SE"] = [self.stderror]
@@ -103,6 +139,11 @@ class _SurveyEstimator:
             self.number_psus = np.unique(psu).size if psu.size > 1 else samp_weight.size
             self.number_strata = 1
         elif psu.size > 1:
+            # if type object ("O") change it to str, otherwise np.unique will fail
+            if stratum.dtype == "O":
+                stratum = stratum.astype(str)
+            if psu.dtype == "O":
+                psu = psu.astype(str)
             self.number_psus = np.unique([stratum, psu], axis=1).shape[1]
             self.number_strata = np.unique(stratum).size
         else:
@@ -133,7 +174,7 @@ class _SurveyEstimator:
         domain: Optional[np.newaxis] = None,
         as_factor: bool = False,
         remove_nan: bool = False,
-    ) -> Any: #Union[dict[StringNumber, DictStrNum], DictStrNum, Number]:
+    ) -> Any:  # Union[dict[StringNumber, DictStrNum], DictStrNum, Number]:
         """Computes the parameter point estimates
 
         Args:
@@ -304,10 +345,7 @@ class TaylorEstimator(_SurveyEstimator):
 
     @staticmethod
     def _variance_stratum_within(
-        y_score_s: np.ndarray,
-        number_psus_in_s: np.ndarray,
-        psu_s: np.ndarray,
-        ssu_s: np.ndarray,
+        y_score_s: np.ndarray, number_psus_in_s: np.ndarray, psu_s: np.ndarray, ssu_s: np.ndarray,
     ) -> float:
 
         variance = 0.0
@@ -372,8 +410,7 @@ class TaylorEstimator(_SurveyEstimator):
         as_factor: bool = False,
         remove_nan: bool = False,
     ) -> tuple[
-        Union[dict[StringNumber, Any], np.ndarray],
-        Union[dict[StringNumber, Any], np.ndarray],
+        Union[dict[StringNumber, Any], np.ndarray], Union[dict[StringNumber, Any], np.ndarray],
     ]:
 
         if self.parameter == "ratio" and x is None:
@@ -591,7 +628,6 @@ class TaylorEstimator(_SurveyEstimator):
                     lower_ci = {}
                     upper_ci = {}
                     coef_var = {}
-                    # breakpoint()
                     for level in self.variance[key]:
                         stderror[level] = math.sqrt(self.variance[key][level])
                         lower_ci[level] = self.point_est[key][level] - t_quantile * stderror[level]
@@ -676,6 +712,8 @@ class TaylorEstimator(_SurveyEstimator):
             else:
                 self.fpc = fpc
 
+        self.as_factor = as_factor
+
         if by is None:
             self._estimate(
                 y=y_temp,
@@ -727,3 +765,69 @@ class TaylorEstimator(_SurveyEstimator):
                 self.stderror[b] = by_est.stderror
                 self.lower_ci[b] = by_est.lower_ci
                 self.upper_ci[b] = by_est.upper_ci
+
+    def to_dataframe(self, col_names: Optional(list) = None,) -> pd.DataFrame:
+
+        if self.point_est is None:
+            raise AssertionError("No estimates yet. Must first run estimate().")
+        elif col_names is None:
+            if self.parameter == "proportion" or self.as_factor:
+                col_names = [
+                    "_parameter",
+                    "_domain",
+                    "_level",
+                    "_estimate",
+                    "_stderror",
+                    "_lci",
+                    "_uci",
+                    "_cv",
+                    "_deff",
+                ]
+            else:
+                col_names = [
+                    "_parameter",
+                    "_domain",
+                    "_estimate",
+                    "_stderror",
+                    "_lci",
+                    "_uci",
+                    "_cv",
+                    "_deff",
+                ]
+            if self.deff == {}:
+                col_names.pop()
+            if self.domains is None:
+                col_names.pop(1)
+        else:
+            ncols = len(col_names)
+            if self.deff is not None and self.as_factor is not None and ncols != 9:
+                raise AssertionError("col_names must have 9 values")
+            if self.deff is None and self.as_factor is not None and ncols != 8:
+                raise AssertionError("col_names must have 8 values")
+            if self.deff is not None and self.as_factor is None and ncols != 8:
+                raise AssertionError("col_names must have 8 values")
+            if self.deff is None and self.as_factor is None and ncols != 7:
+                raise AssertionError("col_names must have 7 values")
+
+        if self.deff == {}:
+            est_df = dict_to_dataframe(
+                col_names,
+                self.point_est,
+                self.stderror,
+                self.lower_ci,
+                self.upper_ci,
+                self.coef_var,
+            )
+        else:
+            est_df = dict_to_dataframe(
+                col_names,
+                self.point_est,
+                self.stderror,
+                self.lower_ci,
+                self.upper_ci,
+                self.coef_var,
+                self.deff,
+            )
+        est_df.iloc[:, 0] = self.parameter
+
+        return est_df
