@@ -1,4 +1,5 @@
 from samplics.weighting.adjustment import SampleWeight
 from samplics.weighting.replicates import ReplicateWeight
 
-_all__ = ["SampleWeight", "ReplicateWeight"]
+
+__all__ = ["SampleWeight", "ReplicateWeight"]
